//+------------------------------------------------------------------+
//|                                                StochasticRVI.mq4 |
//|                                                                  |
//| Stochastic RVI                                                   |
//|                                                                  |
//| Algorithm taken from book                                        |
//|     "Cybernetics Analysis for Stock and Futures"                 |
//| by John F. Ehlers                                                |
//|                                                                  |
//|                                              contact@mqlsoft.com |
//|                                          http://www.mqlsoft.com/ |
//+------------------------------------------------------------------+
//------------------------------------------------------------------
#property copyright "www.forex-tsd.com"
#property link      "www.forex-tsd.com"
//------------------------------------------------------------------

#property indicator_separate_window
#property indicator_buffers 2
#property indicator_color1  LimeGreen
#property indicator_color2  Red
#property indicator_width1  3
#property indicator_width2  3
#property indicator_minimum 0
#property indicator_maximum 1

//
//
//
//
//

extern string TimeFrame       = "Current time frame";
extern int    Length          = 8;

extern bool   alertsOn        = true;
extern bool   alertsOnCurrent = false;
extern bool   alertsMessage   = true;
extern bool   alertsSound     = false;
extern bool   alertsNotify    = false;
extern bool   alertsEmail     = false;
extern string soundfile       = "alert2.wav"; 

//
//
//
//
//

double UpH[];
double DnH[]; 
double StocRVI[];
double Trigger[];
double RVI[];
double Value1[];
double Value2[];
double Value3[];
double trend[];

//
//
//
//
//

//
//
//
//
//

string indicatorFileName;
int    timeFrame;
bool   returnBars;
bool   calculateValue;

//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
//
//
//
//
//

int init() 
{
    IndicatorBuffers(8);
    SetIndexBuffer(0,UpH); SetIndexStyle(0,DRAW_HISTOGRAM);
    SetIndexBuffer(1,DnH); SetIndexStyle(1,DRAW_HISTOGRAM); 
    SetIndexBuffer(2,StocRVI);
    SetIndexBuffer(3,Trigger);
    SetIndexBuffer(4,RVI);
    SetIndexBuffer(5,Value1);
    SetIndexBuffer(6,Value2);
    SetIndexBuffer(7,Value3);
    
      //
      //
      //
      //
      //
     
        indicatorFileName = WindowExpertName();
        calculateValue    = (TimeFrame=="calculateValue"); if (calculateValue) return(0);
        returnBars        = (TimeFrame=="returnBars");     if (returnBars)     return(0);
        timeFrame         = stringToTimeFrame(TimeFrame); 
        
      //
      //
      //
      //
      //
     
     IndicatorShortName(timeFrameToString(timeFrame)+"  StochRVI ("+Length+")");
return (0);
}

//
//
//
//
//

int start() 
{
   int counted_bars=IndicatorCounted();
   int i,r,shift,limit;

   if(counted_bars<0) return(-1);
   if(counted_bars>0) counted_bars--;
         limit = MathMin(Bars-counted_bars,Bars-1);
         if (returnBars) { UpH[0] = limit+1; return(0); }
   
   //
   //
   //
   //
   //
   
   if (calculateValue || timeFrame==Period())
   {
      if (ArrayRange(trend,0)!=Bars) ArrayResize(trend,Bars); 
      for(i=limit, r=Bars-i-1; i>=0; i--,r++)
      { 
   
        Value1[i] = ((Close[i]-Open[i])+ 2.0 * (Close[i+1]-Open[i+1])+ 2.0 * (Close[i+2]-Open[i+2])+(Close[i+3]-Open[i+3]))/6.0;
        Value2[i] = ((High[i] -Low[i]) + 2.0 * (High[i+1] -Low[i+1]) + 2.0 * (High[i+2] - Low[i+2])+(High[i+3] - Low[i+3]))/ 6.0;
      
        //
        //
        //
        //
        //
      
        double Num   = 0.0;
        double Denom = 0.0;
        for (int count = 0; count < Length; count++) 
        {
           Num   += Value1[i+count];
           Denom += Value2[i+count];
        }        
        if (Denom != 0.0) 
        {
          RVI[i] = Num / Denom;
        } 
        else 
        {
          RVI[i] = 0.0;
        }
        
        //
        //
        //
        //
        //
        
        double rvi = RVI[i];
        double hh  = rvi; 
        double ll  = rvi;
        for (int k = 0; k < Length; k++)
        {
           double tmp = RVI[i+k];
           hh = MathMax(hh, tmp);
           ll = MathMin(ll, tmp);
        }
           Value3[i] = 0.0;
           if (hh != ll) 
           { 
             Value3[i]  = (rvi - ll) / (hh - ll); 
           }
           
           //
           //
           //
           //
           //
           
           StocRVI[i] = (4.0 * Value3[i]     + 3.0 * Value3[i+1]+2.0 * Value3[i+2] + Value3[i+3]) / 10.0;
           StocRVI[i] = 2.0  * (StocRVI[i]   - 0.5);
           Trigger[i] = 0.96 * (StocRVI[i+1] + 0.02);
           
           //
           //
           //
           //
           //
           
           UpH[i] = EMPTY_VALUE;
           DnH[i] = EMPTY_VALUE;
           trend[r] = trend[r-1];
           if (StocRVI[i]>Trigger[i]) trend[r] = 1;  
           if (StocRVI[i]<Trigger[i]) trend[r] =-1;
           if (trend[r] == 1) UpH[i] = 1;
           if (trend[r] ==-1) DnH[i] = 1;       
      }
      manageAlerts();
      return(0);
      }
      
      //
      //
      //
      //
      //
   
      limit = MathMax(limit,MathMin(Bars-1,iCustom(NULL,timeFrame,indicatorFileName,"returnBars",0,0)*timeFrame/Period()));
      if (ArrayRange(trend,0)!=Bars) ArrayResize(trend,Bars); 
      for(i=limit, r=Bars-i-1; i>=0; i--,r++)
      {
         int y = iBarShift(NULL,timeFrame,Time[i]);
            StocRVI[i] = iCustom(NULL,timeFrame,indicatorFileName,"calculateValue",Length,2,y);
            Trigger[i] = iCustom(NULL,timeFrame,indicatorFileName,"calculateValue",Length,3,y);
            UpH[i] = EMPTY_VALUE;
            DnH[i] = EMPTY_VALUE;
            
            trend[r] = trend[r-1];
            if (StocRVI[i]>Trigger[i]) trend[r] = 1;  
            if (StocRVI[i]<Trigger[i]) trend[r] =-1;
            if (trend[r] == 1) UpH[i] = 1;
            if (trend[r] ==-1) DnH[i] = 1;      
         
      }
      manageAlerts();  
return(0);         
}


//+-------------------------------------------------------------------
//|                                                                  
//+-------------------------------------------------------------------
//
//
//
//
//

string sTfTable[] = {"M1","M5","M15","M30","H1","H4","D1","W1","MN"};
int    iTfTable[] = {1,5,15,30,60,240,1440,10080,43200};

//
//
//
//
//

int stringToTimeFrame(string tfs)
{
   tfs = stringUpperCase(tfs);
   for (int i=ArraySize(iTfTable)-1; i>=0; i--)
         if (tfs==sTfTable[i] || tfs==""+iTfTable[i]) return(MathMax(iTfTable[i],Period()));
                                                      return(Period());
}
string timeFrameToString(int tf)
{
   for (int i=ArraySize(iTfTable)-1; i>=0; i--) 
         if (tf==iTfTable[i]) return(sTfTable[i]);
                              return("");
}

//
//
//
//
//

string stringUpperCase(string str)
{
   string   s = str;

   for (int length=StringLen(str)-1; length>=0; length--)
   {
      int char = StringGetChar(s, length);
         if((char > 96 && char < 123) || (char > 223 && char < 256))
                     s = StringSetChar(s, length, char - 32);
         else if(char > -33 && char < 0)
                     s = StringSetChar(s, length, char + 224);
   }
   return(s);
}

//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
//
//
//
//
//

void manageAlerts()
{
   if (!calculateValue && alertsOn)
   {
      if (alertsOnCurrent)
           int whichBar = 0;
      else     whichBar = 1; whichBar = Bars-iBarShift(NULL,0,iTime(NULL,timeFrame,whichBar))-1;
      if (trend[whichBar] != trend[whichBar-1])
      {
         if (trend[whichBar] == 1) doAlert(whichBar,"up");
         if (trend[whichBar] ==-1) doAlert(whichBar,"down");
      }         
   }
}   

//
//
//
//
//

void doAlert(int forBar, string doWhat)
{
   static string   previousAlert="nothing";
   static datetime previousTime;
   string message;
   
      if (previousAlert != doWhat || previousTime != Time[forBar]) {
          previousAlert  = doWhat;
          previousTime   = Time[forBar];

          //
          //
          //
          //
          //

          message =  StringConcatenate(Symbol()," ",timeFrameToString(timeFrame)," at ",TimeToStr(TimeLocal(),TIME_SECONDS)," StochRvi ",doWhat);
             if (alertsMessage) Alert(message);
             if (alertsEmail)   SendMail(StringConcatenate(Symbol(),"StochRvi "),message);
             if (alertsNotify)  SendNotification(StringConcatenate(Symbol(), Period() ," StochRvi " +" "+message));
             if (alertsSound)   PlaySound("alert2.wav");
      }
}


